<?php

namespace Drupal\Tests\fences_block\Functional;

use Drupal\Tests\block\Functional\BlockTestBase;

/**
 * This class provides methods specifically for testing something.
 *
 * @group fences_block
 */
class FencesBlockFunctionalBlockStarkTest extends BlockTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'system',
    'block',
    'filter',
    'test_page_test',
    'help',
    'block_test',
    'fences',
    'fences_block',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'stark';

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Setup is done in parent class already!
  }

  public function testPoweredByBlock(): void {
    $session = $this->assertSession();

    // Place the block:
    $this->placeSystemBlock('system_powered_by_block', 'sidebar_second', TRUE, [
      'wrapper' => [
        'element' => 'span',
      ],
      'label' => [
        'element' => 'strong',
      ],
      'content' => [
        'element' => 'em',
      ],
    ]);

    // See core/modules/block/tests/src/Functional/BlockSystemBrandingTest.php.
    // Set a site slogan.
    $this->config('system.site')
      ->set('slogan', 'Community plumbing')
      ->save();

    // Debug block settings:
    // $this->drupalGet('/admin/structure/block/manage/poweredbydrupal');

    // Test the frontpage:
    $this->drupalGet('<front>');
    $session->pageTextContains('Test page text.');

    $session->pageTextContains('Powered by Drupal');
    $session->elementExists('css', '#block-poweredbydrupal');
    $session->elementExists('css', 'span#block-poweredbydrupal');
    $session->elementExists('css', 'span#block-poweredbydrupal.test-wrapper-class');
    $session->elementExists('css', '#block-poweredbydrupal > em.test-content-class');
    $session->elementExists('css', '#block-poweredbydrupal > strong');
    $session->elementExists('css', 'aside > div > span#block-poweredbydrupal.test-wrapper-class > em.test-content-class > span > a');
    $session->elementTextEquals('css', '#block-poweredbydrupal em.test-content-class', 'Powered by Drupal');
  }


  // DOES NOT WORK BY DESIGN, SEE FencesBlockFunctionalBlockStarkDysfunctional.php
  // and https://www.drupal.org/project/fences_block/issues/3304737
  // /**
  // * Test system branding block.
  // */
  // public function testSystemBrandingBlockWithLabel(): void {
  //   $session = $this->assertSession();

  //   // Place the block:
  //   $this->placeSystemBlock('system_branding_block', 'header', TRUE, []);

  //   // Set a site slogan.
  //   // See core/modules/block/tests/src/Functional/BlockSystemBrandingTest.php.
  //   $this->config('system.site')
  //     ->set('slogan', 'Community plumbing')
  //     ->save();

  //   // Debug block settings:
  //   $this->drupalGet('/admin/structure/block/manage/sitebranding');

  //   // Test the frontpage:
  //   $this->drupalGet('<front>');
  //   $session->pageTextContains('Test page text.');

  //   $session->pageTextContains('Community plumbing');
  //   $session->elementExists('css', '#block-sitebranding');
  //   $session->elementExists('css', 'article#block-sitebranding');
  //   $session->elementAttributeContains('css', '#block-sitebranding', 'class', 'test-wrapper-class');
  //   $session->elementExists('css', '#block-sitebranding > h3.test-label-class');
  //   $session->elementTextEquals('css', '#block-sitebranding > h3.test-label-class', 'Site branding');
  //   $session->elementExists('css', '#block-sitebranding > article.test-content-class');
  // }

  /**
   * Test system page title block.
   */
  public function testPageTitleBlock(): void {
    $session = $this->assertSession();

    // Place the block:
    $this->placeSystemBlock('page_title_block', 'content', FALSE, []);

    // Debug block settings:
    // $this->drupalGet('/admin/structure/block/manage/pagetitle');

    // Test on frontpage:
    $this->drupalGet('<front>');
    $session->pageTextContains('Test page text.');

    $session->elementExists('css', '#block-pagetitle');
    $session->elementExists('css', 'article#block-pagetitle');
    $session->elementAttributeContains('css', 'article#block-pagetitle', 'class', 'test-wrapper-class');
    // We don't display a label:
    $session->elementNotExists('css', '#block-pagetitle h3.test-label-class');
    $session->elementExists('css', 'article#block-pagetitle.test-wrapper-class em.test-content-class');
  }

  // DOES NOT WORK BY DESIGN, SEE FencesBlockFunctionalBlockStarkDysfunctional.php
  // and https://www.drupal.org/project/fences_block/issues/3304737
  // /**
  //  * Test system main content block.
  //  */
  // public function testSystemMainBlock(): void {
  //   $session = $this->assertSession();

  //   // Place the block:
  //   $this->placeSystemBlock('system_main_block', 'content', FALSE, []);

  //   // Debug block settings:
  //   $this->drupalGet('/admin/structure/block/manage/mainpagecontent');

  //   // Test on frontpage:
  //   $this->drupalGet('<front>');
  //   $session->pageTextContains('Test page text.');

  //   $session->elementExists('css', '#block-mainpagecontent');
  //   $session->elementExists('css', 'article#block-mainpagecontent');
  //   $session->elementAttributeContains('css', '#block-mainpagecontent', 'class', 'test-wrapper-class');
  //   // We don't display a label:
  //   // $session->elementExists('css', '#block-mainpagecontent > h3.test-label-class');
  //   // $session->elementTextEquals('css', '#block-mainpagecontent > h3.test-label-class', 'Test page');
  //   $session->elementExists('css', '#block-mainpagecontent > article.test-content-class');
  // }

  /**
   * Tests multiple blocks placed on the same page in different regions.
   */
  public function testSystemBlocksCombined(): void {
    $session = $this->assertSession();

    // Place the block:
    $this->placeSystemBlock('system_powered_by_block', 'sidebar_second', TRUE, []);
    $this->placeSystemBlock('system_branding_block', 'header', TRUE, []);
    $this->placeSystemBlock('page_title_block', 'content', TRUE, []);
    $this->placeSystemBlock('system_main_block', 'content', TRUE, []);

    // Debug block settings:
    // $this->drupalGet('/admin/structure/block/manage/mainpagecontent');

    // Test on frontpage:
    $this->drupalGet('<front>');
    $session->pageTextContains('Test page text.');

    // TODO: Add further tests for all blocks!
    // Expected to work:
    $session->elementExists('css', '#block-poweredbydrupal');
    $session->elementExists('css', 'article#block-poweredbydrupal');

    // Expected NOT to work:
    $session->elementExists('css', '#block-sitebranding');
    $session->elementNotExists('css', 'article#block-sitebranding');

    // Expected to work:
    $session->elementExists('css', '#block-pagetitle');
    $session->elementExists('css', 'article#block-pagetitle');

    // Expected to work:
    $session->elementExists('css', '#block-mainpagecontent');
    $session->elementExists('css', 'article#block-mainpagecontent');
  }

  /**
   * Helper function to place a system block with certain settings.
   *
   * @param string $block_id The system block id.
   * @param string $region The region where the block is placed.
   * @param bool $labelVisible Display the label?
   * @param array $settings Override default test settings.
   */
  protected function placeSystemBlock(string $block_id, string $region = 'content', bool $labelVisible = TRUE, array $settings = []): void {
    $page = $this->getSession()->getPage();
    // Edit block:
    $this->drupalGet('/admin/structure/block/add/' . $block_id . '/' . $this->defaultTheme);
    $page->fillField('edit-region', $region);

    $formValues = [
      'settings[fences][sections][wrapper][element]' => isset($settings['wrapper']['element']) ? $settings['wrapper']['element'] : 'article',
      'settings[fences][sections][wrapper][classes]' => isset($settings['wrapper']['classes']) ? $settings['wrapper']['classes'] : 'test-wrapper-class',
      'settings[fences][sections][label][element]' => isset($settings['label']['element']) ? $settings['label']['element'] : 'h3',
      'settings[fences][sections][label][classes]' => isset($settings['label']['classes']) ? $settings['label']['classes'] : 'test-label-class',
      'settings[fences][sections][content][element]' => isset($settings['content']['element']) ? $settings['content']['element'] : 'em',
      'settings[fences][sections][content][classes]' => isset($settings['content']['classes']) ? $settings['content']['classes'] : 'test-content-class',
    ];
    if ($labelVisible) {
      $formValues['settings[label_display]'] = 'visible';
    }

    $this->submitForm($formValues, 'Save block');
  }

}
